﻿unit Main;

interface

uses
	Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
	Dialogs, StdCtrls, ComCtrls, Registry, ActnList;

type
	TPersonality = (pIDE, pDelphi, pDelphiDotNet, pCBuilder, pCSharp, pVB,
		pKnownPackages, pKnownAssemblies);

type
	TForm1 = class(TForm)
		SettingPageControl: TPageControl;
		KnownIDEPackagesTabSheet: TTabSheet;
		RegPathDescLabel: TLabel;
		KnownIDEPackagesListView: TListView;
		ApplyButton: TButton;
		ActionList1: TActionList;
		ApplyAction: TAction;
    DelphiTabSheet: TTabSheet;
    DelphiDotNetTabSheet: TTabSheet;
    CBuilderTabSheet: TTabSheet;
    CSharpTabSheet: TTabSheet;
    VBTabSheet: TTabSheet;
    DelphiListView: TListView;
    DelphiDotNetListView: TListView;
    CSharpListView: TListView;
    VBListView: TListView;
    CBuilderListView: TListView;
    KnownPackagesTabSheet: TTabSheet;
    KnownAssembliesTabSheet: TTabSheet;
    KnownPackagesListView: TListView;
    KnownAssembiesListView: TListView;
		procedure ApplyActionExecute(Sender: TObject);
		procedure ApplyActionUpdate(Sender: TObject);
		procedure PackagesListViewChange(Sender: TObject; Item: TListItem;
			Change: TItemChange);
		procedure FormDestroy(Sender: TObject);
		procedure FormCreate(Sender: TObject);
	private
		FRegistry: TRegistry;
    FListViews: array[TPersonality] of TListView;
    FKnownIDEPackagesChanged: array[TPersonality] of Boolean;
	protected
		procedure ReadKnownIDEPackages(Item: TPersonality);
		procedure WriteKnownIDEPackages(Item: TPersonality);
		procedure CheckListView(ListView: TListView; const Items: array of string; Check: Boolean);
		procedure ReadValuesToListView(const Known: string; const Removed: string; ListView: TListView);
		procedure WriteValuesFromListView(const Known: string; const Removed: string; ListView: TListView);
	end;

var
	Form1: TForm1;

const
	BDSRootKey = '\Software\Borland\BDS\4.0';

	KnownIDEPackagesKey: array[TPersonality] of string = (
    '\Software\Borland\BDS\4.0\Known IDE Packages',
    '\Software\Borland\BDS\4.0\Known IDE Packages\Delphi',
    '\Software\Borland\BDS\4.0\Known IDE Packages\DelphiDotNet',
    '\Software\Borland\BDS\4.0\Known IDE Packages\CBuilder',
    '\Software\Borland\BDS\4.0\Known IDE Packages\CSharp',
		'\Software\Borland\BDS\4.0\Known IDE Packages\VB',
		'\Software\Borland\BDS\4.0\Known Packages',
		'\Software\Borland\BDS\4.0\Known Assemblies');

	RemovedIDEPackagesKey: array[TPersonality] of string = (
		'\Software\Borland\BDS\4.0\Known IDE Packages (removed)',
		'\Software\Borland\BDS\4.0\Known IDE Packages (removed)\Delphi',
		'\Software\Borland\BDS\4.0\Known IDE Packages (removed)\DelphiDotNet',
		'\Software\Borland\BDS\4.0\Known IDE Packages (removed)\CBuilder',
		'\Software\Borland\BDS\4.0\Known IDE Packages (removed)\CSharp',
		'\Software\Borland\BDS\4.0\Known IDE Packages (removed)\VB',
		'\Software\Borland\BDS\4.0\Known Packages (removed)',
		'\Software\Borland\BDS\4.0\Known Assemblies (removed)');

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var
	I: TPersonality;
begin
	FListViews[pIDE] := KnownIDEPackagesListView;
	FListViews[pDelphi] := DelphiListView;
	FListViews[pDelphiDotNet] := DelphiDotNetListView;
	FListViews[pCBuilder] := CBuilderListView;
	FListViews[pCSharp] := CSharpListView;
	FListViews[pVB] := VBListView;
	FListViews[pKnownPackages] := KnownPackagesListView;
	FListViews[pKnownAssemblies] := KnownAssembiesListView;
	for I := Low(TPersonality) to High(TPersonality) do
		FListViews[I].Tag := Ord(I);

	FRegistry := TRegistry.Create;
	FRegistry.RootKey := HKEY_CURRENT_USER;

	for I := Low(TPersonality) to High(TPersonality) do
		ReadKnownIDEPackages(I);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
	FRegistry.Free;
end;

procedure TForm1.PackagesListViewChange(Sender: TObject; Item: TListItem; Change: TItemChange);
begin
	if Change = ctState then
	begin
		FKnownIDEPackagesChanged[TPersonality((Sender as TListView).Tag)] := True;
	end
end;

procedure TForm1.ApplyActionUpdate(Sender: TObject);
var
  I: Boolean;
begin
  for I in FKnownIDEPackagesChanged do
  begin
    if I then
    begin
      ApplyAction.Enabled := True;
      Exit;
    end;
  end;
	ApplyAction.Enabled := False;
end;

procedure TForm1.ApplyActionExecute(Sender: TObject);
var
  I: TPersonality;
begin
  for I := Low(TPersonality) to High(TPersonality) do
  begin
    if FKnownIDEPackagesChanged[I] then
    begin
      FKnownIDEPackagesChanged[I] := False;
      WriteKnownIDEPackages(I);
    end;
  end;
end;

procedure TForm1.ReadKnownIDEPackages(Item: TPersonality);
begin
	ReadValuesToListView(
    KnownIDEPackagesKey[Item],
    RemovedIDEPackagesKey[Item],
    FListViews[Item]);

	FKnownIDEPackagesChanged[Item] := False;
end;

procedure TForm1.WriteKnownIDEPackages(Item: TPersonality);
begin
	if KnownIDEPackagesListView.Enabled then
	begin
    WriteValuesFromListView(
      KnownIDEPackagesKey[Item],
      RemovedIDEPackagesKey[Item],
      FListViews[Item]);
	end
end;

procedure TForm1.CheckListView(ListView: TListView; const Items: array of string; Check: Boolean);
var
	Item: TListItem;
	ItemName: string;
begin
	for Item in ListView.Items do
	begin
		for ItemName in Items do
		begin
			if AnsiSameText(Item.Caption, ItemName) then
			begin
				Item.Checked := Check;
				Break;
			end;
    end;
	end;
end;

procedure TForm1.ReadValuesToListView(const Known: string; const Removed: string; ListView: TListView);
var
	Names: TStrings;
	Name, Desc: string;
	Item: TListItem;
begin
	ListView.Clear;
	try
		Names := TStringList.Create;
		try
			FRegistry.Access := KEY_READ;
			if FRegistry.OpenKeyReadOnly(Known) then
			begin
				FRegistry.GetValueNames(Names);
				for Name in Names do
				begin
					Desc := FRegistry.ReadString(Name);
					Item := ListView.Items.Add;
					Item.Caption := Name;
					Item.SubItems.Add(Desc);
					Item.Checked := True;
				end;
			end;
			if FRegistry.OpenKeyReadOnly(Removed) then
			begin
				FRegistry.GetValueNames(Names);
				for Name in Names do
				begin
					Desc := FRegistry.ReadString(Name);
					Item := ListView.Items.Add;
					Item.Caption := Name;
					Item.SubItems.Add(Desc);
					Item.Checked := False;
				end;
			end;
		finally
			Names.Free;
		end
	except
	on ERegistryException do
	begin
		ListView.Clear;
		ListView.Enabled := False;
	end;
	end;
end;

procedure TForm1.WriteValuesFromListView(const Known, Removed: string; ListView: TListView);
var
	Names: TStrings;
	Name, Desc: string;
	Item: TListItem;
begin
	try
		Names := TStringList.Create;
		try
			FRegistry.Access := KEY_ALL_ACCESS;
			if FRegistry.OpenKey(Known, True) then
			begin
				FRegistry.GetValueNames(Names);
				for Name in Names do
				begin
					FRegistry.DeleteValue(Name); {Result: Boolean -> 無視}
				end;
				for Item in ListView.Items do
				begin
					if Item.Checked then
					begin
						Name := Item.Caption;
						Desc := Item.SubItems[0];
						FRegistry.WriteString(Name, Desc);
					end;
				end;
			end;
			if FRegistry.OpenKey(Removed, True) then
			begin
				FRegistry.GetValueNames(Names);
				for Name in Names do
				begin
					FRegistry.DeleteValue(Name); {Result: Boolean -> 無視}
				end;
				for Item in ListView.Items do
				begin
					if not Item.Checked then
					begin
						Name := Item.Caption;
						Desc := Item.SubItems[0];
						FRegistry.WriteString(Name, Desc);
					end;
				end;
			end;
		finally
			Names.Free;
		end
	except
	on E: ERegistryException do
		Application.ShowException(E);
	end;
end;

end.

